﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using GetFactorsCount;

namespace GetFactorsCountTests
{
    [TestFixture]
    public class GetFactorsCountTests
    {
        private Factors _factors;

        [SetUp]
        public void Setup()
        {
            _factors = new Factors();
        }

        /// <summary>
        /// Passing 0 should return zero
        /// </summary>
        [Test]
        public void GetFactors_isZeroWhenPassedZero_ReturnsTrue()
        {
            int count = _factors.GetFactorsCount(0);
            Assert.AreEqual(0, count, "count should be 0 when passed a zero");
        }

        /// <summary>
        /// Generate a random negative number (between lowest minimum integer (i.e. −2,147,483,648) and -1)
        /// Passing a negative number should return zero
        /// </summary>
        [Test]
        public void GetFactors_isZeroWhenPassedNegativeNumber_ReturnsTrue()
        {
            Random rand = new Random();

            int count = _factors.GetFactorsCount(rand.Next(int.MinValue, -1));
            Assert.AreEqual(0, count, "count should be 0 when passed a negative number");
        }

        /// <summary>
        /// Passing 1 should return 1
        /// </summary>
        [Test]
        public void GetFactors_is1WhenPassed1_ReturnsTrue()
        {
            int count = _factors.GetFactorsCount(1);
            Assert.AreEqual(1, 1, "should be 1");
        }

        /// <summary>
        /// Passing 6 should return 4
        /// Passing 125 should return 4
        /// Passing 2048 should return 12
        /// </summary>
        [Test]
        public void GetFactors_passingThreeKnownIntegers_ReturnsTrue()
        {
            int count = _factors.GetFactorsCount(6);
            Assert.AreEqual(4, count, "6 should only have 4 factors");

            count = _factors.GetFactorsCount(125);
            Assert.AreEqual(4, count, "125 should only have 4 factors");

            count = _factors.GetFactorsCount(2048);
            Assert.AreEqual(12, count, "2048 should only have 12 factors");
        }

        /// <summary>
        /// Passing MaxInteger (2,147,483,647) should only return 2
        /// </summary>
        [Test]
        public void GetFactors_passingMaxInteger_ReturnsTrue()
        {
            int count = _factors.GetFactorsCount(int.MaxValue);
            Assert.AreEqual(2, count, "MaxInteger should only have 2 factors");
        }

        [TearDown]
        public void TearDown()
        {
            _factors = null;
        }
    }
}
